.bk.card {
  border: 1px solid rgba(0,0,0,.125);
  border-radius: 0.25rem;
}
.bk.accordion {
  border: 1px solid rgba(0,0,0,.125);
}
.bk.card-header {
  align-items: center;
  background-color: rgba(0, 0, 0, 0.03);
  border-radius: 0.25rem;
  display: inline-flex;
  justify-content: start;
  width: 100%;
}
.bk.accordion-header {
  align-items: center;
  background-color: rgba(0, 0, 0, 0.03);
  border-radius: 0;
  display: flex;
  justify-content: start;
  width: 100%;
}
.bk.card-button {
  background-color: transparent;
  margin-left: 0.5em;
}
.bk.card-header-row {
  position: relative !important;
}
.bk.card-title {
  align-items: center;
  font-size: 1.4em;
  font-weight: bold;
  overflow-wrap: break-word;
}
.bk.card-header-row > .bk {
  overflow-wrap: break-word;
  text-align: center;
}
